clear all
macro drop _all
cap log close
log using iGRU_and_CoateConlinRFandINT_Estimation, replace text

set more off

use coateconlinaer.dta, clear
global percip rainfall snowfall


***Linear Probability regression***
gen lnf=0
capture program drop lfsur
program lfsur
version 11
args lnf theta1 theta2 sigma1 sigma2 rho
tempvar z
local y1 $ML_y1
local y2 $ML_y2
gen double `z'=(`y1'-`theta1')^2/`sigma1'^2-2*`rho'*(`y1'-`theta1')*(`y2'-`theta2')/`sigma1'/`sigma2'+(`y2'-`theta2')^2/`sigma2'^2
quietly replace `lnf'=-ln(2*_pi*`sigma1'*`sigma2'*sqrt(1-`rho'^2))-`z'/2/(1-`rho'^2) 
quietly replace lnf=`lnf'
end

local forX alc bars citych nonecty weekend $percip avetemp avetemp2 summer perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa evoters
local agaX alc bars citych nonecty weekend $percip avetemp avetemp2 summer perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa evoters
	   
ml model lf lfsur (f: ffor =`forX') (a: fagainst = `agaX') (sigma_f:) (sigma_a:) (rho:),nopreserve
ml maximize

ml score fforgrad fagagrad fsigfgrad fsigagrad frhograd
foreach list of local forX {
    gen gradf`list' = `list'*fforgrad
	global gradlist_sur $gradlist_sur gradf`list'
}
global gradlist_sur $gradlist_sur fforgrad
foreach list of local agaX {
    gen grada`list' = `list'*fagagrad
	global gradlist_sur $gradlist_sur grada`list'
	}
global gradlist_sur $gradlist_sur fagagrad fsigfgrad fsigagrad frhograd
matrix sur_Hinv=-e(V)	//Save Heissian 
gen sur_lnf=lnf			//Save lnf
drop lnf

***Improved Group rule-utilitarian model***
gen lnf=0

capture program drop vote
program define vote
version 6

args lnf be xe ce ve the sig
tempvar v th b c x lnuj bvw eps neps

quietly gen double `v' = exp(`ve')/(1+exp(`ve'))
quietly gen double `th' = (`the')^2
quietly gen double `b' = exp(`be') 
quietly gen double `x' = exp(`xe') 
quietly gen double `c' = exp(`ce')

quietly gen double `bvw'=exp(lngamma(`v'*`th'))*exp(lngamma((1-`v')*`th'))/exp(lngamma(`th'))   //B(v,theta)

quietly gen double `lnuj' = (`th'*`v'-1)*ln($ML_y1)+(`th'*(1-`v')-1)*ln($ML_y2)+ ///
(`th'*`v'/2)*ln(`v'*`x')+(`th'*(1-`v')/2)*ln((1-`v')*`b')-ln(`bvw')- ///
`th'*ln($ML_y1*(`v'*`x')^(1/2)+$ML_y2*((1-`v')*`b')^(1/2))

quietly gen double `eps' = ln(`c')+(`th'+1)*ln(((1-`v')*`b')^(1/2)+(`v'*`x')^(1/2))+ln(`bvw') ///
+ln($ML_y1*(`v'*`x')^(1/2)+$ML_y2*((1-`v')*`b')^(1/2))-(`th'*`v'/2)*ln(`v'*`x') ///
-(`th'*(1-`v')/2)*ln((1-`v')*`b') - ln(`x'*`b')

quietly gen double `neps' =-.5*ln(2*_pi*`sig'^2)-`eps'^2/(2*`sig'^2)

quietly replace `lnf'= `neps'+`lnuj'
quietly replace lnf=`lnf'

end

local demosocial perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa
local grp_cX weekend $percip avetemp avetemp2 summer 
local grp_bX alc bars citych nonecty 
local grp_xX alc bars citych nonecty 
local grp_vX  `demosocial'

gen evoters100000 = evoters/100000

ml model lf vote (b: ffor fagainst=`grp_bX') (x:`grp_xX')  (c:`grp_cX',noconst) (v:`grp_vX') (th:evoters100000)(sig:) 

ml init b:alc = .1653569 b:bars = -.2827705 b:citych=.8499666 b:nonecty=.258794 b:_cons=-1.433741 ///
x:alc = .0829321 x:bars = -.6340133 x:citych = .6209855 x:nonecty=.4015235 x:_cons = -.1189187 ///
c:weekend = .154527 c:rainfall = .0011705 c:snowfall = .0302446 c:avetemp = .0017763 c:avetemp2 = 0 c:summer=-.0440842 ///
v:perbap = -.9611609 v:evotov50=-1.02 v:fsmale = 3.996503 v:fblack = .4211758 v:fmex = -.1279951 v:fhouseo = .9980478 ///
v:acctyrlg = .078255 v:housepri = 0 v:dmsa = -.5253597 v:_cons = .9113363 ///
th:_cons=3.415578 ///
sig:_cons = .5938734

ml maximize

ml score grp_bgrad grp_xgrad grp_cgrad grp_vgrad grp_wgrad grp_siggrad

foreach list of local grp_bX {
    gen grad_grp_b`list' = `list'*grp_bgrad
	global gradlist_group $gradlist_group grad_grp_b`list'
}
global gradlist_group $gradlist_group grp_bgrad
foreach list of local grp_xX {
    gen grad_grp_x`list' = `list'*grp_xgrad
	global gradlist_group $gradlist_group grad_grp_x`list'
}
global gradlist_group $gradlist_group grp_xgrad
foreach list of local grp_cX {
    gen grad_grp_c`list' = `list'*grp_cgrad
	global gradlist_group $gradlist_group grad_grp_c`list'
}
/*global gradlist_group $gradlist_group grp_cgrad*/
foreach list of local grp_vX {
    gen grad_grp_v`list' = `list'*grp_vgrad
	global gradlist_group $gradlist_group grad_grp_v`list'
}
global gradlist_group $gradlist_group grp_vgrad

gen grad_grp_wevoters = evoters10000*grp_wgrad
global gradlist_group $gradlist_group grad_grp_wevoters grp_wgrad grp_siggrad

matrix group_Hinv=-e(V)
gen group_lnf = lnf
drop lnf

***Intensity model***
gen lnf=0

capture program drop vote
program define vote
args lnf be xe ce ae ve we
tempvar v w b x c a lnuj bvw eps neps
quietly gen double `v' = exp(`ve')
quietly gen double `w' = exp(`we')
quietly gen double `b' = exp(`be')
quietly gen double `x' = exp(`xe')
quietly gen double `c' = exp(`ce')
quietly gen double `a' = exp(`ae')
quietly gen double `bvw'=exp(lngamma(`v'))*exp(lngamma(`w'))/exp(lngamma(`v'+`w'))
quietly gen double `lnuj' = (`v'-1)*ln($ML_y1)+(`w'-1)*ln($ML_y2)+ ///
`v'*ln(`x')+`w'*ln(`b')-ln(`bvw')-(`v'+`w')*ln($ML_y1*`x'+$ML_y2*`b')
quietly gen double `eps' = ln($ML_y1*`x'+$ML_y2*`b')-ln(`a'*`x'*`b')+ln(`c')
quietly gen double `neps' =-.5*ln(2*_pi)-`eps'^2/(2)
quietly replace `lnf'= `neps'+`lnuj'
quietly replace lnf=`lnf'
end

constraint define 1 [x]_cons=[b]_cons+0.8142215
constraint define 2 [x]_cons=[b]_cons-[x]nonecty+[b]nonecty+1.422203
constraint define 3 [x]_cons=[b]_cons-[x]citych+[b]citych+0.33943
constraint define 4 [x]_cons=[b]_cons-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.4215914
constraint define 5 [x]_cons=[b]_cons-[x]bars+[b]bars+1.62515
constraint define 6 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]nonecty+[b]nonecty+1.57311
constraint define 7 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]citych+[b]citych+0.4341106
constraint define 8 [x]_cons=[b]_cons-[x]bars+[b]bars-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.829558
constraint define 9 [x]_cons=[b]_cons-[x]alc+[b]alc+0.6877292
constraint define 10 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]nonecty+[b]nonecty+0.3597384
constraint define 11 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]citych+[b]citych+0.1407262
constraint define 12 [x]_cons=[b]_cons-[x]alc+[b]alc-[x]citych+[b]citych-[x]nonecty+[b]nonecty+0.1639242

local bX alc bars citych nonecty
local xX alc bars citych nonecty
local cX weekend $percip avetemp avetemp2 summer
local aX perbap evotov50 fsmale fblack fmex evoters
local vX perbap evotov50 fsmale fblack fmex fhouseo housepri acctyrlg dmsa

ml model lf vote (b: ffor fagainst=`bX') (x:`xX') (c:`cX',noconst) (a: `aX', noconst) (v:`vX') (w:), constraint(10 12) nopreserve

ml maximize

ml score its_bgrad its_xgrad its_cgrad its_agrad its_vgrad its_wgrad

foreach list of local bX {
    gen grad_its_b`list' = `list'*its_bgrad
	global gradlist_intensity $gradlist_intensity grad_its_b`list'
}
global gradlist_intensity $gradlist_intensity its_bgrad
foreach list of local xX {
    gen grad_its_x`list' = `list'*its_xgrad
	global gradlist_intensity $gradlist_intensity grad_its_x`list'
}
global gradlist_intensity $gradlist_intensity its_xgrad
foreach list of local cX {
    gen grad_its_c`list' = `list'*its_cgrad
	global gradlist_intensity $gradlist_intensity grad_its_c`list'
}
/*global gradlist_intensity $gradlist_intensity grp_cgrad*/
foreach list of local aX {
    gen grad_its_a`list' = `list'*its_agrad
	global gradlist_intensity $gradlist_intensity grad_its_a`list'
}
/*global gradlist_intensity $gradlist_intensity its_agrad*/
foreach list of local vX {
    gen grad_its_v`list' = `list'*its_vgrad
	global gradlist_intensity $gradlist_intensity grad_its_v`list'
}
global gradlist_intensity $gradlist_intensity its_vgrad
global gradlist_intensity $gradlist_intensity its_wgrad
matrix intensity_Hinv=-e(V)
gen intensity_lnf=lnf

log close
